#include "sprox_desfire_i.h"

#ifndef SPROX_DESFIRE_PCSC

/*
 **************************************************************************************************
 *
 *  Exported functions
 *
 *************************************************************************************************
 */
SPROX_API_FUNC(Desfire_SelectCid) (SPROX_PARAM  BYTE cid)
{
  SPROX_DESFIRE_GET_CTX();
  ctx->tcl_cid = cid;
  return DF_OPERATION_OK;
}

SPROX_API_FUNC(Desfire_IsoWrapping) (SPROX_PARAM  BYTE mode)
{
  SPROX_DESFIRE_GET_CTX();
  
  switch (mode)
  {
    case DF_ISO_WRAPPING_OFF    : break;
    case DF_ISO_WRAPPING_CARD   : break;

    case DF_ISO_WRAPPING_READER :
    default                     : return DFCARD_LIB_CALL_ERROR;    
  }

  ctx->iso_wrapping = mode;
  return DF_OPERATION_OK;
}


/*
 **************************************************************************************************
 *
 *  Command : "low" level command<->response transfert
 *
 *************************************************************************************************
 */

SPROX_API_FUNC(Desfire_Command) (SPROX_PARAM  BOOL fast, WORD accept_len, BYTE accept_status)
{
  SPROX_RC   status;
  BYTE    card_status;
  WORD    len = DESFIRE_BUFFER_SIZE;
  SPROX_DESFIRE_GET_CTX();

//  D(printf("S: "));
//  D(dump_buffer(ctx->buffer, ctx->buflen);
//  D(printf("\n"));

  /* Communicate the prepared buffer to the PICC and get its response. */
  if (fast)
  {
    status = SPROX_API_CALL(TclA_ExchangeDF) (SPROX_PARAM_P  ctx->tcl_cid, ctx->buffer, ctx->buflen, ctx->buffer, &len);
  } else
  {
    status = SPROX_API_CALL(TclA_Exchange) (SPROX_PARAM_P  5, ctx->tcl_cid, 0xFF, ctx->buffer, ctx->buflen, ctx->buffer, &len);
  }
  if (status != MI_OK)
  {
    /* Error: could not exchange info blocks with the PICC. */
    return status;
  }

  /* Remember received size */
  ctx->buflen = len;

//  D(printf("R: "));
//  D(dump_buffer(ctx->buffer, ctx->buflen);
//  D(printf("\n"));

  /* Check whether the PICC's response consists of at least one byte.
     If the response is empty, we can not even determine the PICC's status.
     Therefore an empty response is always a length error. */
  if (len < 1)
  {
    /* Error: block with inappropriate number of bytes received from the PICC. */
    return DFCARD_WRONG_LENGTH;
  }

  /*  Check the status byte received from the PICC.
     Depending on which of the flags (sgbACCEPT_OPERATION_OK and sgbACCEPT_ADDITIONAL_FRAME)
     are set in the bExpectedStatusFlags parameter, we accept either the status byte 
     sgbOPERATION_OK or the status byte sgbDF_ADDITIONAL_FRAME or both of them. */
  card_status = ctx->buffer[INF + 0];

  if (!(((card_status == DF_OPERATION_OK) && (accept_status & ACCEPT_OPERATION_OK)) ||
        ((card_status == DF_ADDITIONAL_FRAME) && (accept_status & ACCEPT_ADDITIONAL_FRAME))))
  {
    /* Error: error or unexpected status received from the PICC. */
    return (DFCARD_ERROR - card_status);
  }

  /* Finally check the length of the response. It must be > 0 (already checked) and
     <= sgbMAX_INFO_FRAME_SIZE in all cases.
     If bExpectedResponseLength is != 0, we expect a response with exactly that length.
     If bExpectedResponseLength == 0, we allow any response length > 0 and 
     <= sgbMAX_INFO_FRAME_SIZE. */
  if (((!fast) && (len > MAX_INFO_FRAME_SIZE)) || ((accept_len != 0) && (accept_len != len)))
  {
    /* Error: block with inappropriate number of bytes received from the PICC. */
    return DFCARD_WRONG_LENGTH;
  }

  /* Return the status code from the PICC without any prefix code. */
  return status;
}

#endif