#ifndef _SPROX_DESFIRE_I_H_
#define _SPROX_DESFIRE_I_H_

#ifdef WIN32
  #include <windows.h>
  #include <tchar.h>
  #define SPROX_DESFIRE_LIB __declspec( dllexport )
  #define SPROX_DESFIRE_API __cdecl
#endif

#ifndef SPROX_DESFIRE_PCSC

  /*
   * SpringProx API compliant library
   * --------------------------------
   */
   
    #define SPROX_RC                     SWORD

  #ifndef SPROX_API_REENTRANT
    #define SPROX_API_FUNC_T(a, b)       SPROX_DESFIRE_LIB a     SPROX_DESFIRE_API SPROX_ ## b
    #define SPROX_API_FUNC(a)            SPROX_DESFIRE_LIB SWORD SPROX_DESFIRE_API SPROX_ ## a
    #define SPROX_DESFIRE_GET_CTX()      SPROX_DESFIRE_CTX_ST *ctx = &desfire_ctx;
    #define SPROX_DESFIRE_GET_CTX_V()    SPROX_DESFIRE_CTX_ST *ctx = &desfire_ctx;
    #define SPROX_API_CALL(a)            SPROX_ ## a
    #define SPROX_PARAM
    #define SPROX_PARAM_V                void
    #define SPROX_PARAM_P
    #define SPROX_PARAM_PV
    
    #include "springprox.h"
    #include "sprox_desfire.h"
    
    SPRINGPROX_LIB SWORD SPRINGPROX_API SPROX_TclA_ExchangeDF(BYTE cid, const BYTE send_buffer[], WORD send_len, BYTE recv_buffer[], WORD *recv_len);
    
  #else
    #define SPROX_API_FUNC_T(a, b)       SPROX_DESFIRE_LIB a     SPROX_DESFIRE_API SPROXx_ ## b
    #define SPROX_API_FUNC(a)            SPROX_DESFIRE_LIB SWORD SPROX_DESFIRE_API SPROXx_ ## a
    #define SPROX_DESFIRE_GET_CTX()      SPROX_DESFIRE_CTX_ST *ctx = desfire_get_ctx(rInst); if (ctx == NULL) return MI_INVALID_READER_CONTEXT;
    #define SPROX_DESFIRE_GET_CTX_V()    SPROX_DESFIRE_CTX_ST *ctx = desfire_get_ctx(rInst); if (ctx == NULL) return;
    #define SPROX_API_CALL(a)            SPROXx_ ## a
    #define SPROX_PARAM                  SPROX_INSTANCE rInst, 
    #define SPROX_PARAM_V                SPROX_INSTANCE rInst
    #define SPROX_PARAM_P                rInst,
    #define SPROX_PARAM_PV               rInst
    
    #include "springprox_ex.h"
    #include "sprox_desfire_ex.h"
    
    SPRINGPROX_LIB SWORD SPRINGPROX_API SPROXx_TclA_ExchangeDF(SPROX_INSTANCE rInst, BYTE cid, const BYTE send_buffer[], WORD send_len, BYTE recv_buffer[], WORD *recv_len);
     
  #endif
  
#else
  
  /*
   * PC/SC API compliant library
   * ---------------------------
   */
    
    #define SPROX_RC                     LONG

    #define SPROX_API_FUNC_T(a, b)       SPROX_DESFIRE_LIB a     SPROX_DESFIRE_API SCard ## b
    #define SPROX_API_FUNC(a)            SPROX_DESFIRE_LIB LONG  SPROX_DESFIRE_API SCard ## a
    #define SPROX_DESFIRE_GET_CTX()      SPROX_DESFIRE_CTX_ST *ctx = desfire_get_ctx(hCard); if (ctx == NULL) return SCARD_E_INVALID_HANDLE;
    #define SPROX_DESFIRE_GET_CTX_V()    SPROX_DESFIRE_CTX_ST *ctx = desfire_get_ctx(hCard); if (ctx == NULL) return;
    #define SPROX_API_CALL(a)            SCard ## a
    #define SPROX_PARAM                  SCARDHANDLE hCard, 
    #define SPROX_PARAM_V                SCARDHANDLE hCard
    #define SPROX_PARAM_P                hCard,
    #define SPROX_PARAM_PV               hCard
   
    #include "pcsc_desfire.h"


#endif


/* DES and 3-DES ciphering must be embedded in this library */
/* -------------------------------------------------------- */

typedef struct
{
  DWORD encrypt_subkeys[32];
  DWORD decrypt_subkeys[32];  
} DES_CTX_ST;

typedef struct
{
  DES_CTX_ST key1_ctx;
  DES_CTX_ST key2_ctx;
  DES_CTX_ST key3_ctx;
} TDES_CTX_ST;

typedef union
{
  DES_CTX_ST  des;
  TDES_CTX_ST tdes;
} DES_TDES_CTX_ST;

void DES_Init(DES_CTX_ST *context, const BYTE key[8]);

void DES_Encrypt(DES_CTX_ST *context, BYTE inoutbuf[8]);
void DES_Decrypt(DES_CTX_ST *context, BYTE inoutbuf[8]);

void DES_Encrypt2(DES_CTX_ST *context, BYTE outbuf[8], const BYTE inbuf[8]);
void DES_Decrypt2(DES_CTX_ST *context, BYTE outbuf[8], const BYTE inbuf[8]);

void TDES_Init(TDES_CTX_ST *context, const BYTE key1[8], const BYTE key2[8], const BYTE key3[8]);

void TDES_Encrypt(TDES_CTX_ST *context, BYTE inoutbuf[8]);
void TDES_Decrypt(TDES_CTX_ST *context, BYTE inoutbuf[8]);

void TDES_Encrypt2(TDES_CTX_ST *context, BYTE outbuf[8], const BYTE inbuf[8]);
void TDES_Decrypt2(TDES_CTX_ST *context, BYTE outbuf[8], const BYTE inbuf[8]);

WORD ComputeIso14443ACrc(BYTE crc[2], const BYTE buffer[], WORD size);


BOOL BuildIsoApdu(const BYTE native_buffer[], WORD native_buflen, BYTE iso_buffer[], WORD *iso_buflen, BOOL use_reader_wrapping);


#include "desfire.h"


/*
 * Functions from the desfire_crypto module
 */
SPROX_API_FUNC_T(void, Desfire_CipherMAC)  (SPROX_PARAM  BYTE data[], WORD blocks, const BYTE key[16], BYTE result[4]);
SPROX_API_FUNC_T(void, Desfire_CipherMACb) (SPROX_PARAM  BYTE data[], DWORD length, const BYTE key[16], BYTE result[4]);
SPROX_API_FUNC_T(void, Desfire_CipherRecv) (SPROX_PARAM  BYTE data[], WORD blocks, const BYTE key[16]);
SPROX_API_FUNC_T(void, Desfire_CipherSend) (SPROX_PARAM  BYTE data[], WORD blocks, const BYTE key[16]);

/*
 * Internal module functions
 */
SPROX_API_FUNC(Desfire_Command)     (SPROX_PARAM  BOOL fast, WORD accept_len, BYTE accept_status);
SPROX_API_FUNC(Desfire_ReadDataEx)  (SPROX_PARAM  BYTE read_command, BYTE file_id, BYTE comm_mode, DWORD from_offset, DWORD item_count, DWORD item_size, BYTE * data, DWORD * done_size);
SPROX_API_FUNC(Desfire_WriteDataEx) (SPROX_PARAM  BYTE write_command, BYTE file_id, BYTE comm_mode, DWORD from_offset, DWORD size, const BYTE data[]);
SPROX_API_FUNC(Desfire_ModifyValue) (SPROX_PARAM  BYTE modify_command, BYTE file_id, BYTE comm_mode, SDWORD amount);




/* Flags for DesFireCard_Command */
#define INF 0
#define ACCEPT_OPERATION_OK      0x01
#define ACCEPT_ADDITIONAL_FRAME  0x02


#define	INVALIDATE_AUTHENTICATION	          0xFF


#undef D
#define D(x)

void dump_buffer(const BYTE * buffer, WORD len);


#define DESFIRE_BUFFER_SIZE 4096

typedef struct
{
  BYTE tcl_cid;
  BYTE iso_wrapping;
  BYTE auth_key;
  BYTE session_key[16];

  WORD buflen;
  BYTE buffer[DESFIRE_BUFFER_SIZE];

  BYTE cipher_buffer[DESFIRE_BUFFER_SIZE+8];

  DES_TDES_CTX_ST cipher_context;
} SPROX_DESFIRE_CTX_ST;

#ifndef SPROX_DESFIRE_PCSC
  #ifndef SPROX_API_REENTRANT
    extern SPROX_DESFIRE_CTX_ST desfire_ctx;
  #else
    SPROX_DESFIRE_CTX_ST *desfire_get_ctx(rInst);
  #endif
#else
    SPROX_DESFIRE_CTX_ST *desfire_get_ctx(hCard);
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#endif
