#ifndef _CARDAPPZ_H_
#define _CARDAPPZ_H_

/*
 * Application level
 * -----------------
 */
#include "cardappz_types.h"
#include "cardappz_errors.h"

#include "cardappz_corpid.h"
#include "cardappz_purse.h"
#include "cardappz_transport.h"

/* Blank the card */
int Blank(CARDAPPZ_CTX_ST *ctx);

/* Format the card */
int Format(CARDAPPZ_CTX_ST *ctx);

/* Compute a diversified key */
int ComputeDivKey(CARDAPPZ_CTX_ST *ctx, unsigned char DiversifiedKey[16], const unsigned char MasterKey[16]);

/* Compute a signature */
int ComputeSignature(CARDAPPZ_CTX_ST *ctx, unsigned char Signature[8], unsigned char Data[], unsigned long Size, const unsigned char SignKey[16]);

/*
 * Card specific stuff
 * -------------------
 */
#include "cardappz_desfire.h"
#include "cardappz_jaycos.h"

/*
 * Lower level glue
 * ----------------
 */
#include "des.h"
#include "md5.h"


/*
 * Finally, the ADT used everywhere
 * --------------------------------
 */

typedef enum { NOTRACE, HIGHLEVEL, LOWLEVEL, TRACE, DEBUG, DUMPALL, DUMPMORE } cardappz_trace;

#define CARD_TYPE_14443_A   0x40
#define CARD_TYPE_14443_B   0x80
#define CARD_TYPE_14443_4   0x20
#define CARD_TYPE_MIFARE_ST 0x10

#define CARD_MIFARE_UL      (CARD_TYPE_14443_A |                       0)
#define CARD_DESFIRE        (CARD_TYPE_14443_A | CARD_TYPE_14443_4   | 0)
#define CARD_MIFARE_1K      (CARD_TYPE_14443_A | CARD_TYPE_MIFARE_ST | 0)
#define CARD_MIFARE_4K      (CARD_TYPE_14443_A | CARD_TYPE_MIFARE_ST | 1)
#define CARD_JAYCOS         (CARD_TYPE_14443_B | CARD_TYPE_14443_4   | 0)
#define CARD_SR176          (CARD_TYPE_14443_B |                       0)
#define CARD_CTS256B        (CARD_TYPE_14443_B |                       1)


struct cardappz_ctx_st
{
  cardappz_trace trace;

  unsigned char card_type;

  union
  {
    struct
    {
      unsigned char atq[2];
      unsigned char sak[1];
      unsigned char snr[12];
      unsigned char snrlen;
      unsigned char ats[31];
      unsigned char atslen;
    } a;
    struct
    {
      unsigned char atq[11];
    } b;
  } card_ident;

  union
  {
    struct
    {
      unsigned char auth_status;
      unsigned char auth_key;
      unsigned char auth_with_sam;
      TDES_CTX_ST   tdes_ctx;
    } desfire;
  } card_context;

  struct
  {
    unsigned char found;
    unsigned char slot;
    unsigned char atr[32];
    unsigned char atrlen;
    unsigned char auth_status;
    unsigned char auth_key;
    TDES_CTX_ST   tdes_ctx;   
  } desfire_sam;
};


#endif
