#include "cardappz.h"


int DesfireSelectPurse(CARDAPPZ_CTX_ST * ctx)
{
  int rc;

  unsigned char key[16];

  /* Select the application directory */
  rc = DesfireSelectApplication(ctx, PURSE_DESFIRE_AID);
  if (rc)
    return rc;

  /* Compute the diversified key 1 */
  rc = ComputeDivKey(ctx, key, PURSE_MASTER_KEY1);
  if (rc)
    return rc;

  /* Get authenticated with key 1 */
  rc = DesfireAuthenticate(ctx, 1, key);
  if (rc)
    return rc;

  return CARDAPPZ_OK;
}

int DesfireReadPurseBalance(CARDAPPZ_CTX_ST * ctx, unsigned long *Balance)
{
  int rc;
  signed long v;

  rc =
    DesfireGetValue(ctx, PURSE_DESFIRE_FID_BALANCE, DESFIRE_COMM_PLAIN, &v);
  if (rc)
    return rc;

  if (v < 0)
    return CARDAPPZ_CARD_CONTENT_ERROR;

  if (Balance != NULL)
    *Balance = v;
  return CARDAPPZ_OK;
}

int DesfireReadPurseReloadEvent(CARDAPPZ_CTX_ST * ctx, int Index,
                                PURSE_RELOAD_ST * ReloadEntry)
{
  return DesfireReadRecords(ctx, PURSE_DESFIRE_FID_RELOAD,
                            DESFIRE_COMM_PLAIN, Index, 1,
                            (unsigned char *) ReloadEntry, NULL);
}

int DesfireReadPursePurchaseEvent(CARDAPPZ_CTX_ST * ctx, int Index,
                                  PURSE_PURCHASE_ST * PurchaseEntry)
{
  return DesfireReadRecords(ctx, PURSE_DESFIRE_FID_PURCHASE,
                            DESFIRE_COMM_PLAIN, Index, 1,
                            (unsigned char *) PurchaseEntry, NULL);
}

/* New purchase + decrement counter */
int DesfireDebitPurse(CARDAPPZ_CTX_ST * ctx,
                      PURSE_PURCHASE_ST * PurchaseEntry)
{
  int rc;

  if (PurchaseEntry == NULL)
    return CARDAPPZ_INVALID_PARAMETER;

  if (!PurchaseEntry->Debit)
    return CARDAPPZ_INVALID_VALUE;

  /* Decrement balance */
  rc =
    DesfireDebit(ctx, PURSE_DESFIRE_FID_BALANCE, DESFIRE_COMM_PLAIN,
                 PurchaseEntry->Debit);
  if (rc)
    return rc;

  /* Insert new record */
  rc =
    DesfireWriteRecord(ctx, PURSE_DESFIRE_FID_PURCHASE, DESFIRE_COMM_PLAIN,
                       0, sizeof(PURSE_PURCHASE_ST),
                       (unsigned char *) PurchaseEntry);
  if (rc)
    goto rollback;

  /* Commit transaction */
  rc = DesfireCommitTransaction(ctx);
  if (rc)
    return rc;

  return CARDAPPZ_OK;

rollback:
  if (rc != CARDAPPZ_INVALID_VALUE)
    DesfireAbortTransaction(ctx);
  return rc;
}

/* Re-load + increment counter */
int DesfireCreditPurse(CARDAPPZ_CTX_ST * ctx, PURSE_RELOAD_ST * ReloadEntry)
{
  signed long v;
  int rc;
  unsigned char key[16];

  if (ReloadEntry == NULL)
    return CARDAPPZ_INVALID_PARAMETER;

  if (!ReloadEntry->Credit)
    return CARDAPPZ_INVALID_VALUE;

  /* Select the application directory */
  rc = DesfireSelectApplication(ctx, PURSE_DESFIRE_AID);
  if (rc)
    return rc;

  /* Compute the diversified key 0 */
  rc = ComputeDivKey(ctx, key, PURSE_MASTER_KEY0);
  if (rc)
    return rc;

  /* Get authenticated with key 0 */
  rc = DesfireAuthenticate(ctx, 0, key);
  if (rc)
    return rc;

  /* Read current value of counter */
  rc =
    DesfireGetValue(ctx, PURSE_DESFIRE_FID_BALANCE, DESFIRE_COMM_PLAIN, &v);
  if (rc)
    return rc;

  /* New balance will be... */
  ReloadEntry->NewBalance = ReloadEntry->Credit + v;

  /* Increment balance */
  rc =
    DesfireCredit(ctx, PURSE_DESFIRE_FID_BALANCE, DESFIRE_COMM_PLAIN,
                  ReloadEntry->Credit);
  if (rc)
    goto rollback;

  /* Write new record */
  rc =
    DesfireWriteRecord(ctx, PURSE_DESFIRE_FID_RELOAD, DESFIRE_COMM_PLAIN, 0,
                       sizeof(PURSE_RELOAD_ST),
                       (unsigned char *) ReloadEntry);
  if (rc)
    goto rollback;

  /* Commit transaction */
  rc = DesfireCommitTransaction(ctx);
  if (rc)
    return rc;

  return CARDAPPZ_OK;

rollback:
  if (rc != CARDAPPZ_INVALID_VALUE)
    DesfireAbortTransaction(ctx);
  return rc;
}


/* Formatting */

/* ---------- */

int DesfireFormatPurse(CARDAPPZ_CTX_ST * ctx)
{
  int rc;
  unsigned char key[16];

  /* Select application 0 */
  rc = DesfireSelectApplication(ctx, 0);
  if (rc)
    return rc;

  /* Get authenticated with master key */
  rc = DesfireAuthenticate(ctx, 0, DESFIRE_BLANK_KEY);
  if (rc)
    return rc;

  /* Create the purse application with 2 keys */
  rc = DesfireCreateApplication(ctx, PURSE_DESFIRE_AID, 0xE3, 2);
  if (rc)
    return rc;

  /* Select the application */
  rc = DesfireSelectApplication(ctx, PURSE_DESFIRE_AID);
  if (rc)
    return rc;

  /* Formatting is performed with (blank) key 0 */
  rc = DesfireAuthenticate(ctx, 0, DESFIRE_BLANK_KEY);
  if (rc)
    return rc;

  /* Create counter file for balance, min=0, max=50000, initial value = 0, limited credit disabled */
  rc = DesfireCreateValueFile(ctx, PURSE_DESFIRE_FID_BALANCE, DESFIRE_COMM_PLAIN, 0x1F0F, 0, 50000, 0, 0);  /* Plain ; Key 0 = R/W ; Key 1 = R */
  if (rc)
    return rc;

  /* Create re-load history file (4 + 1 records, 16 bytes each) */
  rc = DesfireCreateCyclicRecordFile(ctx, PURSE_DESFIRE_FID_RELOAD, DESFIRE_COMM_PLAIN, 0x1F0F, 16, 5); /* Plain ; Key 0 = R/W ; Key 1 = R */
  if (rc)
    return rc;

  /* Create purchase history file (6 + 1 records, 16 bytes each) */
  rc = DesfireCreateCyclicRecordFile(ctx, PURSE_DESFIRE_FID_PURCHASE, DESFIRE_COMM_PLAIN, 0x110F, 16, 7); /* Plain ; Key 0 = R/W , Key 1 = R+W */
  if (rc)
    return rc;

  /* Compute the diversified key 0 */
  rc = ComputeDivKey(ctx, key, PURSE_MASTER_KEY0);
  if (rc)
    return rc;

  /* Put the new key 0 */
  rc = DesfireChangeKey(ctx, 0, key);
  if (rc)
    return rc;

  /* Get authenticated with (blank) key 1 */
  rc = DesfireAuthenticate(ctx, 1, DESFIRE_BLANK_KEY);
  if (rc)
    return rc;

  /* Compute the diversified key 1 */
  rc = ComputeDivKey(ctx, key, PURSE_MASTER_KEY1);
  if (rc)
    return rc;

  /* Put the new key 1 */
  rc = DesfireChangeKey(ctx, 1, key);
  if (rc)
    return rc;

  /* Done ! */
  return CARDAPPZ_OK;
}
